// FILENAME:	CTF_Inverted.cs
//
// AUTHOR:		Gen. Raven [M.I.B.]
//------------------------------------------------------------------------------
// Stealth Detection - Set to true & players with a "/" in front of their
// will be detected when they join or leave the server.

// Defaulted off since im *hoping* ancients will run this and they already 
// have stealth detect onAdd.
$stealthDetectOnAdd = false;
$stealthDetectOnRemove = true;  //Ancients has none when a player leaves

//------------------------------------------------------------------------------
$missionName = "CTF_Inverted";

$ancientsCTFRules = true;     //AutoHeal after kill or cap, fast/no shutdown pads

$maxFlagCount  = 8;           // no of flags required by a team to end the game
$flagValue     = 5;          // points your team gets for capturing
$carrierValue  = 2;          //  "      "    "    "    " killing carrier
$killPoints    = 1;
$deathPoints   = 1;
$flagTime = 300;

exec("multiplayerStdLib.cs");
exec("CTFstdLib.cs");

function setDefaultMissionOptions()
{
	$server::TeamPlay = true;
	$server::AllowDeathmatch = false;
	$server::AllowTeamPlay = true;	

   // what can the client choose for a team
	$server::AllowTeamRed = false;
	$server::AllowTeamBlue = true;
	$server::AllowTeamYellow = false;
	$server::AllowTeamPurple = true;

   // what can the server admin choose for available teams
   $server::disableTeamRed = true;
   $server::disableTeamBlue = false;
   $server::disableTeamYellow = true;
   $server::disableTeamPurple = false;
}

function onMissionStart()
{
	temperateSounds();
      initGlobalVars();
}

function onMissionLoad()
{
   cdAudioCycle("Cyberntx", "YouGot", "Terror", "Mechsoul", "NewTech"); 
   %count = playermanager::getPlayerCount();
   for(%i=0; %i<%count; %i++)
   {
      %player = playermanager::getPlayerNum(%i);
      %player.stealCount = 0;
   }
}

function player::onAdd(%player)
{
   if(($stealthDetectOnAdd == true)&&(strAlign(1, left, getname(%player))=="/"))
   {
        say(0,0,"<f4>"@getname(%player)@" joined the game","alarm2.wav");
        messagebox(%player,"This server is equipped with an anti-stealth detection system. The slash on your name has been detected. All players have been alerted to your presence.");
   }
   say(%player, 0, "Welcome to CTF Inverted! You can download CTF_Inverted & other missions made by Gen. Raven [M.I.B.] on the MIB Website at www.starsiegemeninblack.cjb.net.");
  
   %player.stealCount = 0;

   //Activate Rock's Admin Pack if its being used
   RocksPlayerOnAdd(%player);
}

function player::onRemove(%player)
{
   if(($stealthDetectOnRemove == true)&&(strAlign(1, left, getname(%player))=="/"))
   {
      say(0,0,"<f4>"@getname(%player)@" left the game","alarm2.wav");
   }
}

//Pad stuff
function ZenHeal::trigger::onEnter(%this, %object)
{
   // tell user pad has been entered
   Zen::onEnter(%this, %object, *IDMULT_CHAT_HEALPAD, true, true);  
}

function ZenHeal::trigger::onContact(%this, %object)
{
   if($ancientsCTFRules==true)
   {
      Zen::work(%this, %object, 1000, 0, 0, false);
   }
   else
   {
      Zen::work(%this, %object, 1000, 0, 0, true);
   }
}

function ZenHeal::trigger::onLeave(%this, %object)
{
   Zen::onLeave(%this, %object); 
}

function ZenAmmo::trigger::onEnter(%this, %object)
{
   // tell user pad has been entered
   Zen::onEnter(%this, %object, *IDMULT_CHAT_AMMOPAD, true, true);  
}

function ZenAmmo::trigger::onContact(%this, %object)
{
   if($ancientsCTFRules==true)
   {
      Zen::work(%this, %object, 0, 25, 0, false);
   }
   else
   {
      Zen::work(%this, %object, 0, 25, 0, true);
   }
}

function ZenAmmo::trigger::onLeave(%this, %object)
{
   Zen::onLeave(%this, %object); 
}

function getSteals(%player)
{
   return %player.stealCount;
}

function getSuccessRate(%player)
{
   %percent = floor(((%player.flagsCollected / %player.stealCount) * 100) + 0.5);
   if((%percent>100) || (%percent<0))
   {
      %percent = 0;
   }
   return %percent @ "%";
}

//From CTFStdLib.cs - Modified functions
//----------------------------------------------------------
// vehicle::onadd()
// vehicle::onDestroyed()
// checkFlagStolen()
// checkFlagRetrieved()
// initScoreBoard()
//----------------------------------------------------------
function vehicle::onAdd(%vehicleId)
{
   %player = playermanager::vehicleIdtoPlayerNum(%vehicleId);
   if($ancientsCTFRules==true)
   {
      $limit = 2;
      %c = getWeaponCount(%vehicleId);
      %i = 0;
      %missles = 0;
      while(%i<%c)
      {
         %id = getWeaponId(%vehicleId,%i);
         if(((%id>=124)&&(%id<=136))||(%id==147)||(%id==142)||(%id==150))
         {
            %missles++;
         }
         %i++;
      }
      if(%missles>$limit)
      {
         healObject(%vehicleId,-50000);
         say(%player,%player,"<f5>THIS SERVER DOES NOT ALLOW MORE THAN <f0>"@$limit@"<f5> MISSLES/MINES/SGUNS/RAD GUNS PER VEHICLE!","rules_violated.wav");
      }
   }

   %team = getTeam(playerManager::vehicleIdToPlayerNum(%vehicleId));
   %color = teamToColor(%team);
   %flagKey = strcat(%color, "FlagCount");

   adjTeamCount(%team, 1);

   //if the flag isn't at the base, but no one has it, correct situation
   if(dataRetrieve(0, %flagKey) && !dataRetrieve(0, strcat(%color, "FlagCarried")))
   {
   	setFlag(%team, true);
   }   
}

function vehicle::onDestroyed(%destroyed,%destroyer)
{
   if($ancientsCTFRules==true)
   {
      if(playermanager::vehicleIdToPlayerNum(%destroyer)!=0)
      {
         schedule("healobject("@%destroyer@",99999);",8);
         schedule("healobject("@%destroyer@",99999);",8);
         schedule("healobject("@%destroyer@",99999);",8);
         schedule("healobject("@%destroyer@",99999);",8);
         schedule("healobject("@%destroyer@",99999);",8);
      }
   }
   %team = getTeam(%destroyed);
   adjTeamCount(%team, -1);
   %teamCount = getTeamPlayerCount(%team);      

   if(%teamCount < 1)
   {
	   setFlag(%team, false);					
   }

   %color = dataRetrieve(%destroyed, "hasFlag");
   %flagTeam = colorToTeam(%color);
   
   // if the destroyed vehicle has a flag, it goes back to it's base
   if (%color != "")
   {
      playerDropsFlag(%destroyed);
      
      // let everyone know this guy drops the flag
      %str =
         *IDMULT_CHAT_SURRENDER_FLAG_1 @
         getName( %destroyed )         @
         *IDMULT_CHAT_SURRENDER_FLAG_2 @ 
         %flagTeam                     @
         *IDMULT_CHAT_SURRENDER_FLAG_3;
      
      %soundFile = "";   
      if(%flagTeam == *IDSTR_TEAM_RED)
      {
         %soundFile = "red_flag_sur.wav";
      }
      else if(%flagTeam == *IDSTR_TEAM_BLUE)
      {
         %soundFile = "blue_flag_sur.wav";
      }
      else if(%flagTeam == *IDSTR_TEAM_YELLOW)
      {
         %soundFile = "yel_flag_sur.wav";
      }
      else
      {
         %soundFile = "purp_flag_sur.wav";
      }   
      
      say( 0, 0, %str, %soundFile );

            
      if($scoringFreeze == false)
      {     
         // destroyer's team gets credit for killing the carrier
         %destroyerTeam = getTeam(%destroyer);  
      
         // only give points if destroyed recovered his/her own flag
         if(%destroyerTeam == colorToTeam(%color))
         {
            %key = strcat(teamToColor(getTeam(%destroyer)), "CarriersKilled");
            dataStore(0, %key, 1 + dataRetrieve(0, %key));
            
            // player gets credit, too
            %player = playerManager::vehicleIdToPlayerNum(%destroyer);
            if(%player != 0)
            {
               %player.carriersKilled = %player.carriersKilled + 1;
            }
         }

         // echo the surrender to the console for logging
         echo(*IDSTR_CONSOLE_CTF_SURRENDER @ " " @ playerManager::vehicleIdToPlayerNum(%destroyed));                        
      }	  
   }
   else
   {
      // not a flag carrier ... is it an enemy?
      if($scoringFreeze == false)
      {
         if(getTeam(%destroyed) != getTeam(%destroyer))
         {
            %player = playerManager::vehicleIdToPlayerNum(%destroyer);
            if(%player != 0)
            {
               %player.genericKills = %player.genericKills + 1;
            }
         }
      }
   }
   
   // enforce the rules
   if(
      (getTeam(%destroyed) == getTeam(%destroyer)) &&
      (%destroyed != %destroyer)
   )
   {
      antiTeamKill(%destroyer);
   }

   vehicle::onDestroyedLog(%destroyed, %destroyer);
   
   // give the death messages...
   %player = playermanager::vehicleIdtoPlayerNum(%destroyed);

   %message = getFancyDeathMessage(getHUDName(%destroyed), getHUDName(%destroyer));
   if(%message != "")
   {
      say( 0, 0, %message);
   }
}

function checkFlagStolen(%team, %vehicle)
{
   %color = teamToColor(%team);
   	
   // you just hit the trigger at an enemy base, is there a flag here?
   %flagCountKey = strcat(%color, "FlagCount");
   %flagCount = dataRetrieve(0, %flagCountKey);
   
   if (%flagCount != 0 || dataRetrieve(%vehicle, "hasFlag") != "") 
   {
      return;
   }

   if (playerManager::vehicleIdToPlayerNum(%vehicle) == 0)
   { 
      return;
   }

   // stop the flag shape from rendering
   setFlag(%team, false);

   // setup the vehicle to carry the flag
   setVehicleSpecialIdentity(%vehicle, true, %color);
   dataStore(%vehicle, "hasFlag", %color);

   // increment flag count so nobody else can pickup this flag
   dataStore(0, %flagCountKey, 1);

   // set global bool for flag of that color
   dataStore(0, strcat(%color, "FlagCarried"), 1); 

   // let everyone know what happened
   %str =
         *IDMULT_CHAT_STEAL_FLAG_1 @
         getName( %vehicle ) @
         *IDMULT_CHAT_STEAL_FLAG_2 @ 
         colorToTeam(%color) @
         *IDMULT_CHAT_STEAL_FLAG_3;
   
   %soundFile = "";
   if(%team == *IDSTR_TEAM_RED)
   {
      %soundFile = "red_flag_stol.wav";
   }
   else if(%team == *IDSTR_TEAM_BLUE)
   {
      %soundFile = "blue_flag_stol.wav";
   }
   else if(%team == *IDSTR_TEAM_YELLOW)
   {
      %soundFile = "yel_flag_stol.wav";
   }
   else
   {
      %soundFile = "purp_flag_stol.wav";
   }

   say( 0, 0, %str, %soundFile );

   // echo the steal to the console for logging
   echo(*IDSTR_CONSOLE_CTF_STEAL @ " " @ playerManager::vehicleIdToPlayerNum(%vehicle));      	                     

   %player = playermanager::vehicleIdToPlayerNum(%vehicle);
   %player.stealCount++;
   
   if($flagTime != 0)
   {      
      // start a hud timer on the flag carrier
      %player = playerManager::vehicleIdToPlayerNum(%vehicle);
      
      %vehicle.timer = schedule("carrierDeath(" @ %vehicle @ ");", $flagTime);
      
      // here we should set the hudtimer
      if(%team == *IDSTR_TEAM_YELLOW)
      {
         %title = *IDMULT_CHAT_YELLOW_FLAG;
      }
      else if(%team == *IDSTR_TEAM_BLUE)
      {
         %title = *IDMULT_CHAT_BLUE_FLAG;
      }
      else if(%team == *IDSTR_TEAM_RED)
      {
         %title = *IDMULT_CHAT_RED_FLAG;
      }
      else if(%team == *IDSTR_TEAM_PURPLE)
      {
         %title = *IDMULT_CHAT_PURPLE_FLAG;
      } 
      
      setHudTimer($flagTime, -1, %title, 1, %player);      
   }
   
   // tell the vehicle where his or her base is
   %playerTeam = getTeam(%vehicle);
   
   if(%playerTeam == *IDSTR_TEAM_YELLOW)
   {
      if($yellowNavPoint != 0)
         setNavMarker($yellowNavPoint, true, %vehicle);
   } else if(%playerTeam == *IDSTR_TEAM_BLUE)
   {
      if($blueNavPoint != 0)
         setNavMarker($blueNavPoint, true, %vehicle);
   } else if(%playerTeam == *IDSTR_TEAM_RED)
   {
      if($redNavPoint != 0)
         setNavMarker($redNavPoint, true, %vehicle);
   } else if(%playerTeam == *IDSTR_TEAM_PURPLE)
   {
      if($purpleNavPoint != 0)
         setNavMarker($purpleNavPoint, true, %vehicle);
   }
}

function checkFlagRetrieved(%team, %vehicle)
{
   %color = teamToColor(%team);

   // you made it back to your own base, do you have any other teams' flags?
   if (dataRetrieve(%vehicle, "hasFlag") == "") 
      { return; }

   // is the player's team flag at his base when he brings back the enemy's flag?
   if (dataRetrieve(0, strcat(%color, "FlagCount")) != 0) 
      { return; }

   // player is no longer carrying the flag
   %hasColor = dataRetrieve(%vehicle, "hasFlag");
   %flagTeam = colorToTeam(%hasColor);
   playerDropsFlag(%vehicle);

   if($scoringFreeze == false)
   {
      // player's team gets credit for stealing the flag
      %collectedKey = strcat(teamToColor(getTeam(%vehicle)), "FlagsCollected");
      %collectedCount = dataRetrieve(0, %collectedKey);
      %collectedCount = %collectedCount + 1;
      dataStore(0, %collectedKey, %collectedCount);
      
      // player gets credit also
      %player = playerManager::vehicleIdToPlayerNum(%vehicle);
      %player.flagsCollected = %player.flagsCollected + 1;

      // echo the capture to the console for logging
      echo(*IDSTR_CONSOLE_CTF_CAPTURE @ " " @ playerManager::vehicleIdToPlayerNum(%vehicle));      	               

      if($ancientsCTFRules==true)
      {
         healObject(%vehicle, 99999);
         healObject(%vehicle, 99999);
         healObject(%vehicle, 99999);
         healObject(%vehicle, 99999);
         healObject(%vehicle, 99999);
      }
      %needMore = $maxFlagCount - %collectedCount;
      if (%needMore <= 0)
      {
         winEvent(getTeam(%vehicle));
      }
   }
  
   // let everyone know what happened
   %str =
      *IDMULT_CHAT_CAPTURE_FLAG_1 @
      getName( %vehicle ) @
      *IDMULT_CHAT_CAPTURE_FLAG_2 @ 
      colorToTeam(%hasColor) @
      *IDMULT_CHAT_CAPTURE_FLAG_3;
   
   %soundFile = "";
   if(%flagTeam == *IDSTR_TEAM_RED)
   {
      %soundFile = "red_flag_cap.wav";
   }
   else if(%flagTeam == *IDSTR_TEAM_BLUE)
   {
      %soundFile = "blue_flag_cap.wav";
   }
   else if(%flagTeam == *IDSTR_TEAM_YELLOW)
   {
      %soundFile = "yel_flag_cap.wav";
   }
   else
   {
      %soundFile = "purp_flag_cap.wav";
   }

   say( 0, 0, %str, %soundFile );      
}

function initScoreBoard()
{
   deleteVariables("$ScoreBoard::PlayerColumn*");
   deleteVariables("$ScoreBoard::TeamColumn*");

   // Player ScoreBoard column headings
   $ScoreBoard::PlayerColumnHeader1 = *IDMULT_SCORE_TEAM;
   $ScoreBoard::PlayerColumnHeader2 = *IDMULT_SCORE_SCORE;
   $ScoreBoard::PlayerColumnHeader3 = *IDMULT_SCORE_FLAGS;
   $ScoreBoard::PlayerColumnHeader4 = "Steals";
   $ScoreBoard::PlayerColumnHeader5 = "Success Rate";
   $ScoreBoard::PlayerColumnHeader6 = *IDMULT_SCORE_CARRIER_KILLS;
   $ScoreBoard::PlayerColumnHeader7 = *IDMULT_SCORE_KILLS;
   $ScoreBoard::PlayerColumnHeader8 = *IDMULT_SCORE_DEATHS;   

   // Player ScoreBoard column functions
   $ScoreBoard::PlayerColumnFunction1 = "getTeam";
   $ScoreBoard::PlayerColumnFunction2 = "getPlayerScore";
   $ScoreBoard::PlayerColumnFunction3 = "getPlayerFlags";
   $ScoreBoard::PlayerColumnFunction4 = "getSteals";
   $ScoreBoard::PlayerColumnFunction5 = "getSuccessRate";
   $ScoreBoard::PlayerColumnFunction6 = "getPlayerCarriersKilled";
   $ScoreBoard::PlayerColumnFunction7 = "getPlayerGenericKills";
   $ScoreBoard::PlayerColumnFunction8 = "getDeaths";


   //Team ScoreBoard column headings
   $ScoreBoard::TeamColumnHeader1 = *IDMULT_SCORE_SCORE;
   $ScoreBoard::TeamColumnHeader2 = *IDMULT_SCORE_PLAYERS;
   $ScoreBoard::TeamColumnHeader3 = *IDMULT_SCORE_FLAGS;
    
   // Team ScoreBoard column functions
   $ScoreBoard::TeamColumnFunction1 = "getTeamScore";
   $ScoreBoard::TeamColumnFunction2 = "getNumberOfPlayersOnTeam";
   $ScoreBoard::TeamColumnFunction3 = "getTeamFlags";

   
   // tell server to process all the scoreboard definitions defined above
   serverInitScoreBoard();   
}